/**
 * A generic auto-updating feed prototype. 
 *
 * @author Andras Sevcsik sevcsik@performgroup.com
 */

'use strict';
/**
 * Generic feed class, supporting live updates.
 */
TS.Modules.Feed = function() {
  /**
   * @type {string} 
   * The feed url. Parameters can be defined like this:
   * http://www.example.com/exampleFeed/{key1}/{key2}
   * where one pair of braces mean the key name, the second
   * one is the value of the key.
   * When using JSONP, it must have a 'callback' field.
   */
  this.url = null;

  /**
   * @type {object}
   * A collection of key/value pairs which will be substituted
   * into the url on every request.
   */
  this.requestData = null;

  /**
   * @type {string}
   * The format of the AJAX request, forwarded to $.ajax dataType.
   */
  this.format = 'jsonp';

  /**
   * Enable automatic refreshing of the feed.
   * If the feed changes, a 'change' event will be triggered
   * on the feed object.
   *
   * @param {number} [updateInterval] the frequency of the updates
   *   in milliseconds. If omitted, the previously set value will
   *   be used.
   */
  this.enableAutoUpdate = function(updateInterval) {
    if (updateInterval) {
      autoUpdateInterval = updateInterval;
    }

    autoUpdateIntervalID = window.setInterval(function() {
      this.reload();
    }.bind(this), autoUpdateInterval);
    return this;
  }.bind(this);

  /** 
   * Turn off auto update.
   */
  this.disableAutoUpdate = function() {
    window.clearInterval(autoUpdateIntervalID);
    return this;
  }.bind(this);

  /**
   * Pull the feed (triggers ready on success).
   * @param {object} [requestData] the data to be substituted into the url.
   *  see the url for available keys.
   * @fires ready
   * @fires error
   */
  this.pull = function(requestData) {
    if (requestData !== undefined) {
      this.requestData = $.extend({}, this.requestData, requestData);
    }
    lastFeedHash = null;
    requestFeed();
    return this;
  }.bind(this);

  /**
   * Pull feed multiple times with different request parameters.
   * 'ready' event will be triggered, when all feeds are loaded.
   *
   * this.data will contain the data of the last successful request. 
   *
   * @param {Array<object>} requestDatas array of request data objects.
   *   This will override fields of this.requestData, so you only have to
   *   add changed fields.
   * @param {Array<Function>|Function} successCallbacks an onsuccess 
   *   function, or an array of functions, each for one requestData.
   * @param {Array<Function>|Function} errorCallbacks same as success
   *   callbacks, but for errors.
   */          
  this.multiPull = function(requestDatas, successCallbacks, errorCallbacks) {
    var count = requestDatas.length;
    var waiting = count;
    var failed = 0;

    if (successCallbacks && 
        successCallbacks.indexOf && 
        requestDatas.length !== successCallbacks.length) {
      throw new Error('successCallbacks must have the same length as requestDatas.');
    }
    
    if (errorCallbacks && 
        errorCallbacks.indexOf && 
        requestDatas.length !== errorCallbacks.length) {
      throw new Error('errorCallbacks must have the same length as requestDatas.');
    }

    var processedResponses = [];
    var rawResponses = [];

    var requestTimeout = 0;
    var onSuccess = function(data, status, jqxhr, index) {
      waiting--;
      requestTimeout && clearTimeout(requestTimeout);
      
      this.data = this.processData(data);
      processedResponses[index] = this.data;
      rawResponses[index] = data;

      var callback = successCallbacks && successCallbacks.indexOf ? 
        successCallbacks[index] :
        successCallbacks;

      if (callback) {
        callback({
          feed: this,
          rawResponse: data,
          processedResponse: this.data,
          multiPullIndex: index
        });
      }

      $(this).trigger('ready', { 
        feed: this, 
        processedResponse: this.data,
        rawResponse: data,
        multiPullIndex: index
      });
      $.trace.info('Feed: feed #' + index + ' loaded');

      if (!waiting) {
        $(this).trigger('multiPullReady', {
          feed: this,
          failed: failed,
          rawResponses: rawResponses,
          processedResponses: processedResponses
        });
        $.trace.info('Feed: all feeds are loaded');
      }
    }.bind(this);

    var onError = function(jqxhr, status, error, index) {
      waiting--;
      failed++;
      requestTimeout && clearTimeout(requestTimeout);
      
      processedResponses[index] = null;
      rawResponses[index] = null;

      var callback = errorCallbacks && errorCallbacks.indexOf ? 
        errorCallbacks[index] :
        errorCallbacks;

      if (callback) {
        callback({
          feed: this,
          error: status,
          httpError: error,
          multiPullIndex: index
        });
      }

      $(this).trigger('error', {
        feed: this,
        error: status,
        httpError: error || 'error',
        multiPullIndex: index
      });
      $.trace.info('Feed: feed #' + index + ' failed to load');

      if (this.parentObject) {
        $(this.parentObject).trigger('error', {
          feed: this,
          failed: failed,
          rawResponses: rawResponses,
          processedResponses: processedResponses
        });
        $.trace.info('Feed: error occured');
      }
    }.bind(this);

    requestDatas.forEach(function(requestData, i, a) {
      var request = $.ajax(applyData(this.url, $.extend({}, this.requestData, requestData)), {
        dataType: this.format,
        beforeSend: this.format === 'jsonp' ? insertJSONPCallback : undefined,
        success: function(a, b, c) { onSuccess(a, b, c, i); },
        error: function(a, b, c) { onError(a, b, c, i); },
        cache: true
      });
      requestTimeout = setTimeout(function(){request.abort("timeout")},TS.Config.feedTimeOut);
    }.bind(this));

    return this;
  };

  /**
   * Pull the feed (triggers change if the content differs)
   * @fires change
   * @fires error
   */
  this.reload = function() {
    requestFeed();
    return this;
  }.bind(this);

  /**
   * @type {Date} The time of the last request.
   */
  this.lastRequestTime = null;

  /**
   * Run the pulled data trough this function before firing 'ready'.
   * By default it does nothing, it should be overwritten on specific
   * feeds if nessecary.
   *
   * @param {object} rawData the data object coming directly from the feed.
   * @returns {bool} if an Error object is returned, it will trigger the 'error' event.
   */
  this.processData = function(rawData) { return rawData; };

  var autoUpdateInterval = 2000;
  var autoUpdateIntervalID;
  var ready = false;
  var rawResponseData = null;
  var lastFeedHash = null; 

  var applyData = function(url, requestData) {
    var ret = url;
    for (var key in requestData) {
      ret = ret.replace('{' + key + '}', encodeURIComponent(requestData[key]));
    }
    return ret;
  }.bind(this);

  var insertJSONPCallback = function(jqXHR, settings) {
    // get jQuery callback name
    var callbackName = settings.url.match(/jQuery[0-9_]+/)[0];
    // delete jQuery callback parameter
    settings.url = settings.url.replace(/[?&]callback=jQuery[0-9_]+/, '');
    // substitute callback name into the url
    settings.url = applyData(settings.url, { callback: callbackName });
  }.bind(this);

  var requestTimeout = 0;
  var onSuccess = function(data) {
    requestTimeout && clearTimeout(requestTimeout);

    this.lastRequestTime = new Date();
    var newFeedHash = hashCode(data);
    rawResponseData = data;
    var processedData = this.processData(rawResponseData);

    if (processedData instanceof Error) {
      $(this).trigger('error', {
        feed: this,
        error: processedData
      });
      return;
    }

    this.data = processedData;

    if (lastFeedHash !== newFeedHash) {
      $(this).trigger('change', {
        feed: this,
        processedResponse: processedData,
        rawResponse: rawResponseData
      });
    }

    $(this).trigger('ready', { 
      feed: this, 
      processedResponse: processedData,
      rawResponse: rawResponseData
    });

    this.data = processedData;
    lastFeedHash = newFeedHash;
  }.bind(this);

  var onError = function(jqXHR, textStatus, errorThrown) {
    requestTimeout && clearTimeout(requestTimeout);
    $.trace.error('Feed ' + this.url + ' failed to load: ' + textStatus);

    $(this).trigger('error', {
      feed: this,
      error: textStatus,
      httpError: errorThrown
    });
    
    if(this.parentObject)
    {
      $(this.parentObject).trigger('error', {
        feed: this,
        error: textStatus,
        httpError: errorThrown
      });
    }
  }.bind(this);

  var requestFeed = function() {
    var request = $.ajax(applyData(this.url, this.requestData), {
      dataType: this.format,
      beforeSend: this.format === 'jsonp' ? insertJSONPCallback : undefined,
      success: onSuccess,
      error: onError,
      cache: true
    });

    // Commented only for testing!
    requestTimeout = setTimeout(function(){request.abort("timeout")},TS.Config.feedTimeOut);
  }.bind(this);

  // source: http://werxltd.com
  var hashCode = function(data) {
    var string = JSON.stringify(data);
    var hash = 0;
    var character;
    if (string.length === 0) return hash;
    for (var i = 0; i < string.length; i++) {
      character = string.charCodeAt(i);
      hash = ((hash<<5)-hash)+character;
      hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
  };
};

$.extend(TS.Modules.Feed, {
  moduleName: 'feed',
  autoInit: false
});

