/**
 * navigationController
 *
 * Tree navigation module
 * @author Andras Sevcsik sevcsik@performgroup.com
 */

'use strict';

TS.Modules.navigationController = {
  moduleName: 'navigationController',
  autoInit: true,
  root: null,

  /** Creates the root navigator */
  init: function() {
    var navigationController = TS.Modules.navigationController;
    var Navigator = TS.Modules.navigationController.Navigator;
    var registeredLinks = [];
    
    /**
     * Scans the DOM for elements with data-navigate attributes,
     * and binds navigation callbacks for them.
     *
     * This must be run every time when new new navigation links
     * are added to the DOM.
     */
    navigationController.updateLinks = function() {

      var links = $('[data-navigate]');
      
      links.each(function updateLinksBindCallbacks(i, e) {
        if (registeredLinks.indexOf(e) === -1) {
          $(e).on('click', function() {
            TS.Config.history.push(e.getAttribute('data-navigate'));
            navigationController.root.navigate(e.getAttribute('data-navigate')); });
          registeredLinks.push(e);
        }
      });
    };
    
    function RootNavigator() {
      this.name = 'ROOT';
      this.activeNavigator = null;
      //this.activeUrl = null;

      /**
       * Jump back one level in the navigation tree.
       */
      var back = function(event) {	  
        if (event.which == TS.KeyCodes.back)
        {
          /*
          console.log(TS.Config.history);
          var target = '/home';
          if(TS.Config.history.length>0 || TS.Config.history===null || !TS.Config.history){
            if(TS.Config.history.length>1){
              TS.Config.history.pop();
            }
            target = TS.Config.history.pop();
            this.navigate(target);
          } else {
            this.navigate('/home');
            window.NetCastBack ? window.NetCastBack() : $.trace.log('If I was a TV I would exit.');
            return true;
          }
          */
          
          // TODO : find out why does this happen - somehow the document.activeElement is the body sometimes during video playing
          if($('#videoObject').length)
          {
            TS.Modules.video.hide();
            return true;
          }

          var arrUrl = this.activeUrl.substr(1).split('/');
          if (this.activeUrl === '/') {
            widgetAPI.sendReturnEvent ? widgetAPI.sendReturnEvent() : $.trace.log('If I was a TV I would exit.');
            return true;
          }
          this.navigate('/'+arrUrl.slice(0,arrUrl.length-1).join('/'));
        }

      }.bind(this);
      this.back = back;

      $(document).keydown(back);
    }

    RootNavigator.prototype = new Navigator();

    /**
     * The mother of all navigators.
     */
    this.root = new RootNavigator();

    $(TS).on('ready', function() {
      // navigate to url
      var hashVars = $.getHash();
      this.root.navigate(hashVars.navigate ? hashVars.navigate : '/');
    }.bind(this));
  }
};


TS.Modules.navigationController.navigatorReady = function(){
	Preloader.hide();
	TS.Modules.navigationController.updateLinks();
};

/**
 * A node of the navigation tree. Each Navigator object should
 * represent a page, or a parent of pages.
 *
 * @param {string} name the name of the navigator. This will be used
 * to match the navigation URLs with the navigator.
 */

TS.Modules.navigationController.Navigator = function(name,caller) {
  this.name = name;

  var navigationController = TS.Modules.navigationController;
  var Navigator = TS.Modules.navigationController.Navigator;
  var children = {};
  var defaultChild, currentChild;
	this.caller = caller;

  /**
   * Add children to the given navigator.
   * @param {Navigator} [...] arguments are a list of child nodes.
   */
  this.addChildren = function() {
    Array.prototype.slice.call(arguments).forEach(
    function argsForEach(e, i, a) {
      if (e instanceof Navigator) {
        children[e.name] = e;
        e.parent = this;
      } else {
        $.trace.error('Navigator ' + this.name + ': Child is not a Navigator instance.');
      }
    }.bind(this));
    return this;
  }.bind(this);

  this.addChild = this.addChildren;

  /**
   * Set a default child. All navigation calls with empty urls
   * will be forwarded to this navigator, if the url doesn't specify a child.
   *
   * @param {Navigator} navigator
   */
  this.setDefault = function(navigator) {
    if (!(navigator instanceof Navigator)) {
      $.trace.error('Navigator ' + this.name + ': Default is not a Navigator instance.');
    }
    defaultChild = navigator;
    return this;
  }.bind(this);

  this.children = children;
  this.defaultChild = defaultChild;

  /**
   * Navigate to a given url. If the url is '', it will navigate to itself,
   * which can cause either to fire an 'open' event, or forward the call to
   * the default child.
   *
   * Event data:
   *  - {string} url: contains the url relative to the Navigator.
   *  - {object} hashVars: contains the key-value pairs extracted from the
   *                       hash part of the url.
   *  - {object} data: additional data passed to the navigate() call
   *  - {Navigator} target: the Navigator which triggers the event.
   *  - {Navigator} [prevNavigator]: only for close events. Points to the 
   *                to-be-closed navigator.
   *
   * @param {string} url an url relative to the navigator.
   * @param {object} [data] data object to pass with the 'open' event
   * @param {bool} [force] if true, the navigation will be done even when the url has not
   *                       changed.
   * @fires close on the previously active navigator.
   * @fires open if there's no default set and url is an empty string.
   * @fires childOpen if the navigate call is forwarded to a child navigator.
   */
  this.navigate = function(url, data, force) {
    
    url && (this.activeUrl = url);
		
		//Preloader hide&show
		if(this.caller)
		{
			var caller = this.caller.moduleName;
			Preloader.show();
			$(this.caller).off('ready',TS.Modules.navigationController.navigatorReady).on('ready',TS.Modules.navigationController.navigatorReady);
		}


    $.trace.info('navigate: ' + this.name + ', url: "' + url + '"');
    if (!force && url === navigationController.activeUrl) {
      return;
    }
    // close active navigator
    if (navigationController.root.activeNavigator) {
      $(navigationController.root.activeNavigator).
        trigger('close', {
          hashvars: $.getHash(url),
          target: this,
          prevNavigator: navigationController.root.activeNavigator,
          url: originalUrl
        });
      $.trace.info('Navigator: close -> ' + navigationController.root.activeNavigator.name);
      if(navigationController.root.activeNavigator.parent){
        $(navigationController.root.activeNavigator.parent).trigger('childClose');
      }
      navigationController.root.activeNavigator = null;
    }

    // remove leading slash from url
    var originalUrl = url;
    if (url.indexOf('/') === 0) {
      url = url.slice(1);
    }
    
    // get first element of the url
    var firstName = url.slice(0, url.indexOf('#') === -1 ? undefined : url.indexOf('#')).split('/')[0];
    // if it's empty, we're the target
    if (!firstName) {
      // if we have a default child, redirect with the rest
      // of the url
      if (defaultChild) {
        // trigger a navigate event if we're forwarding
        $(this).trigger('childOpen', {
          hashVars: $.getHash(url),
          data: data,
          target: this,
          url: originalUrl
        });
        $.trace.info('Navigator: childOpen -> ' + this.name);
        defaultChild.navigate(url.replace(name, ''));
      } else {
        navigationController.root.activeNavigator = this;
        // open ourself
        $(this).trigger('open', {
          hashvars: $.getHash(url),
          data: data,
          target: this,
          url: originalUrl
        });
        $.trace.info('Navigator: open -> ' + this.name);
      }
    } else if (children[firstName]) {
      $(this).trigger('childOpen', {
        hashVars: $.getHash(url),
        data: data,
        target: this,
        url: originalUrl
      });
      $.trace.info('Navigator: close -> ' + this.name);
      children[firstName].navigate(url.replace(firstName, ''), data);
    }
  };
};
