/**
 * @fileOverview videotracker-html5.js
 * HTML5 Video plugin for videotracker.js
 *
 * Usage: Include this file after videotracker.js, and 
 * register the video element with videotracker.trackers.html5Tracker.register().
 * 
 * See 'defaults' field for available options.
 *
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com
 * @version 0.1
 */

jQuery(function($) {
  var provider = new videoTracker.Provider('html5');

  $.extend(provider, {
    defaults: {},
    init: function(options) {
      options = $.extend({}, this.defaults, options);

      var debug = options.debug === undefined ?
        videoTracker.options.debug :
        options.debug;
      this.options = options;
    },
   /**
    * Removes all tracking callbacks from the active video element,
    * and forgets it.
    * This function is overridden after the first call of trackVideo.
    */

    reset: function() {
      video = null;
      videoElement = null;
    },
    /**
     * Bind tracking callbacks to a HTML5 video object. Call this _before_ loading the video!
     * @param {jQuery|HTMLVideoElement}videoElement the video element
     * @param {string} video url
     * @param {string} title Video title
     * @param {string} description
     * @param {string} guid
     * @param {string} adType Possible values: 'preroll', 'postroll' or null
     * @param {bool} [dontTrack] disable tracking for video
     */
    trackVideo: function(videoElement, 
                         url, 
                         title, 
                         description, 
                         guid, 
                         adType, 
                         duration,
                         dontTrack) {
      this.reset();

      var videoElement = this.activeVideoElement = $(videoElement);
      var video = this.activeVideo = new videoTracker.Video(
        title, 
        description, 
        guid,
        duration,
        url, 
        null,
        false,
        false
      );

      var onLoadStart = function() {      
        $(videoTracker).trigger('load', { 
          video: video,
          dontTrack: dontTrack,
        });
      };

      $(videoElement).on('loadstart', onLoadStart);

      var onPlay = function() {
        $(videoTracker).trigger('resume', { 
          video: video,
          dontTrack: dontTrack,
          time: videoElement.prop('currentTime'),
          duration: duration
        });
      }

      $(videoElement).on('play', onPlay);

      var onPause = function() {
        $(videoTracker).trigger('pause', { 
          video: video,
          dontTrack: dontTrack,
          time: videoElement.prop('currentTime'),
          duration: duration
        });
      };

      $(videoElement).on('pause', onPause);

      var onEnded = function() {
        $(videoTracker).trigger('end', { 
          video: video,
          dontTrack: dontTrack,
        });
      };

      $(videoElement).on('ended', onEnded);

      var onProgress = function() {
        $(videoTracker).trigger('progress', { 
          video: video, 
          time: videoElement.prop('currentTime'),
          duration: duration,
          dontTrack: dontTrack
        });
      };

      $(videoElement).on('progress', onProgress);

      this.reset = function() {
        if(videoElement)
        {
          videoElement.off('loadstart', onLoadStart);
          videoElement.off('play', onPlay);
          videoElement.off('pause', onPause);
          videoElement.off('ended', onEnded);
          videoElement.off('progress', onProgress);
        }

        video = null;
        videoElement = null;
      }
    }
  });

  videoTracker.registerProvider(provider);
})