/**
 * @fileOverview videotracker-omniture.js
 *
 * Omniture tracker plugin for videotracker.js
 *
 * Usage: Just include it _after_ videotracker.js, and add /omniture/ to
 * /trackers/ in videotracker init options (or don't).
 *
 * See 'defaults' field for options.
 *
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com>
 * @version 0.1
 */

jQuery(function($)
{
  // Create a new Tracker to register.
  var tracker = new videoTracker.Tracker('omniture');

  /**
   * Maps videoTracker options to omniture trackVars.
   * Only existing fields will be included in returned object.
   *
   * @param object options an options object 
   *  with same structure as videoTracker.options
   * @return object trackVars object
   */
  function getTrackVars(options)
  {
    var trackVars = {};

    if (options.channel) 
    {
      trackVars.videoChannel = 'Guide:' +
        options.channel.genre + ':' +
        options.channel.category + ':' +
        options.channel.name;
    }
    if (options.language) trackVars.languageID = options.language;
    if (options.country) trackVars.countryID = options.country;
    if (options.partnerID) trackVars.partnerID = options.partnerID.toString();
    if (options.siteUrl) trackVars.siteUrl = options.siteUrl;
    if (options.player)
    {
      if (options.player.height !== undefined &&
          options.player.width !== undefined)
        trackVars.playerSize = options.player.height + 'x' +
          options.player.width;

      if (options.player.type)
        trackVars.flashOrHtml = options.player.type == 'html' ?
          'HTML' : 'Flash';
      if (options.player.embedded)
        trackVars.embeddedOrFull = options.player.embedded ? 
          'embedded' : 'full';
      if (options.player.autoPlay !== undefined)
        trackVars.autoPlay = options.player.autoPlay ? 'true' : 'false';
      if (options.player.version)
        trackVars.version = options.player.version;
      if (options.player.name)
        trackVars.type = options.player.name;
    }

    return trackVars;

    /* trackVars for reference
    {
      playerLoaded: false,
      languageID: options.languageID,
      autoPlay: options.autoPlay,
      videoChannel: getChannelString(options.channel),
      partnerID: undefined,
      playerSize: options.playerD,
      countryID: undefined,
      flashOrHtml: 'HTML',
      embeddedOrFull: 'embedded',
      siteUrl: document.location.href,
      type: 'ePlayer',
      version: undefined,
    };
    */
  }


  $.extend(tracker,
  {
    asyncInit: true,
    init: function(options)
    {
      options = $.extend(true, {}, this.defaults, options);
      var debug = options.debug === undefined ? 
        videoTracker.options.debug :
        options.debug;
      this.options = options;      

      // get trackVars from global options
      this.trackVars = getTrackVars(videoTracker.options);

      this.log('starting initialization.');

      // find out s_code.js url
      if (!options.sCodeJsUrl)
      {
        // find our script tag's src
        var sCodeJsUrl = $('script').filter(function()
        {
            return $(this).attr('src') ?
              $(this).attr('src').match('videotracker-omniture.js') :
              false;
        }).attr('src');

        if (!sCodeJsUrl) 
          throw new Error('Couldn\'t determine s_code.js url, and ' +
                          'and it\'s not set in options.');

        // change script name
        sCodeJsUrl = sCodeJsUrl.replace(/videotracker-omniture.js$/, 's_code.js');
      }
      else sCodeJsUrl = options.sCodeJsUrl;

      this.log('fetching s_code.js from ' +
               sCodeJsUrl);

      // pull s_code.js
      $.ajax(sCodeJsUrl,
      {
        dataType: 'text',
        contentType: 'application/javascript',
        success: function(data, textStatus, jqXHR)
        {
          this.log('s_code.js options: ');
          console.log(options);

          // replace config values in s_code.js
          for (var key in options)
          {
            data = data.replace('{{'+key+'}}', options[key]);
          }
          
          /** Dumping the whole s_code.js is silly!
          
            this.log('s_code.js content:\n'+ data);
          */

          // load script
          $('body').append($('<script></script>').html(data));

          this.log('omniture s:');
          console.log(s);

          // bind callbacks to tracker events
          this.bindCallbacks();

          // thanks for all the fish
          this.ready = true;
          $(videoTracker).trigger('trackerready');
        }.bind(this)
      });
    },

    /**
     * Binds videoTracker* events to omniture trackig calls
     */
    bindCallbacks: function()
    {
      var debug = this.options.debug;

      $(videoTracker).on('config', function(event, eventData)
      {
        this.log('Config recevied, data:');
        console.log(event, eventData);

        // Update trackVars from global configuration
        $.extend(true, this.trackVars, getTrackVars(eventData));
      }.bind(this));

      $(videoTracker).on('load', function(event, eventData)
      {
        this.log('Load recevied, data:');
        console.log(event, eventData);
        this.log('s.track, s.Media.open');

        // Set TrackVars
        this.setTrackVars({ playerLoaded: true });

        // Call omniture function
        if (!eventData.dontTrack)
        {
          if (this.options.trackVars.autoPlay === 'false') s.track();
          s.Media.open(eventData.video.title, eventData.video.duration, 
                       this.options.player);
        }
        else 
          this.log('tracking disabled, skipping');
      }.bind(this));
  
      $(videoTracker).on('resume', function(event, eventData)
      {
        this.log('Resume recevied, data:');
        console.log(event, eventData);
        this.log('s.Media.play');

        // Set TrackVars
        if (eventData.video.type == 'preroll')
          this.setTrackVars({ prerollStarted: true } );
        else if (eventData.video.type == 'postroll')
          this.setTrackVars({ postrollStarted: true } );
        else
          this.setTrackVars({});

        // Call omniture function
        if (!eventData.dontTrack)
          s.Media.play(eventData.video.title, eventData.time);
        else 
          this.log('tracking disabled, skipping');
      }.bind(this));

      $(videoTracker).on('pause', function(event, eventData)
      {
        this.log('Pause recevied, data:');
        console.log(event, eventData);
        this.log('s.Media.stop');

        // Set TrackVars
        this.setTrackVars({});

        // Call omniture function
        if (!eventData.dontTrack)
          s.Media.stop(eventData.video.title, eventData.time);
        else 
          this.log('tracking disabled, skipping');
      }.bind(this));

      $(videoTracker).on('end', function(event, eventData)
      {
        this.log('End recevied, data:');
        console.log(event, eventData);
        this.log('s.Media.stop');

        if(!eventData.video)
        {
          this.log('eventData.video missing');
          return false;
        }

        // Set TrackVars
        if (eventData.video.type == 'preroll')
          this.setTrackVars({ prerollEnded: true });
        else if (eventData.video.type == 'postroll')
          this.setTrackVars({ postrollEnded: true });
        else
          this.setTrackVars({ videoEnded: true });

        // Call omniture function
        if (!eventData.dontTrack)
        {
          s.Media.stop(eventData.video.title, 0);
          s.Media.close(eventData.video.title);
        }
        else 
          this.log('tracking disabled, skipping');
      }.bind(this));
    },

    /** 
     * Default values
     *
     * Default values are copied from original s_code.js at r6657 (2012-07-02)
     * and updated s_account value according to ticket PH-123.
     * Undocumented options match s_code.js variables
     * in the format {{KEY}}, in s_code.js.
     *
     */
    defaults: 
    {
      /**
       * URL for s_code.js. If not specified, it will be
       * downloaded from the same directory as /videotracker-omniture.js/.
       * @type string 
       */
      sCodeJsUrl: null,

      /**
       * Player name for Omniture.
       * @type string 
       */
      player: "eplayerHTML5",

      /**
       * If true, configuration from videoTrackerConfig events are ignored.
       * Otherwise, they override the options specified during init.
       * @type bool
       */
      ignoreConfigEvent: false,

      // s_code.js vars
      s_account: 'premiumtveplayermasterdev',
      charSet: 'UTF-8',
      currencyCode: 'GBP',
      trackDownloadLinks: true,
      trackExternalLinks: true,
      trackInlineStats: true,
      linkDownloadFileTypes: 'exe,zip,wav,mp3,mov,mpg,avi,wmv,doc,pdf,xls',
      linkInternalFilters: 'javascript:',
      linkLeaveQueryString: false,
      linkTrackVars: 'None',
      linkTrackEvents: 'None',
      mediaTrackVars: 'events,eVar4,eVar5,eVar6,eVar7,eVar8,eVar11,eVar10,' + 
        'eVar15,eVar12,eVar14,eVar17,prop1,prop2,prop6,prop7,prop9,prop10,' +
        'prop12,prop13,prop18,prop36,prop4',
      mediaTrackEvents: 'event5',
      visitorNamespace: 'premiumtv',
      trackingServer: 'metrics.performgroup.com',
      trackingServerSecure: 'smetrics.performgroup.com',

      trackVars: {} // will be populated in init
    },

    /** 
     * Contains mapping of human-readable field names to eVar* names,
     * based on ePlayer Technical Specifications from Adobe. All fields are 
     * strings.
     */
    trackVarMap:
    {
      playerLoaded: 'event5',
      videEnded: 'event4,event1=94',
      prerollStarted: 'event2',
      prerollEnded: 'event4,event1=6',
      postrollStarted: 'event2', // missing from docs
      postrollEnded: 'event4,event1=11',
      /** like 'fr' */
      languageID: 'eVar4,prop1',
      /** 'auto-play' or 'manual' */
      autoPlay: 'eVar5,prop2',
      /** like 'Sport:Football' */
      videoChannel: 'eVar6,prop6',
      /** like '123' */
      partnerID: 'eVar7,prop7',
      /** like '400x800' */
      playerSize: 'eVar8,prop9',
      /** like 'en_US' */
      countryID: 'eVar11,prop10',
      /** 'Flash' or 'HTML' */
      flashOrHtml: 'eVar10,prop12',
      /** 'embedded' or 'full' */
      embeddedOrFull: 'eVar15,prop13',
      /** like 'www.performgroup.com' */
      siteUrl: 'eVar12,prop18',
      /** like 'ePlayer40' */
      type: 'eVar14,prop36',
      /** like 'ePlayer 2.1 build 1031080512' */
      version: 'eVar17,prop4'
    },
    /**
     * Set trackVars
     *
     * @param {Object} trackVars object containg trackVars, available keys
     *   are in trackVarMap. All missing keys are loaded from 
     *   options.trackVars.
     *
     * @throws 
     */
    setTrackVars: function(trackVars)
    {
      if (!trackVars) trackVars = {};

      s.events = '';

      for (key in this.trackVarMap)
      {
        // get mapped keys
        var keys = this.trackVarMap[key];

        if (!keys) throw new Error('No such key: ' + key);
        keys = keys.split(',');

        // set keys to new value or record event
        for (k2 in keys)
        {
          if (keys[k2].match('event') && 
              (trackVars[key] !== undefined ? trackVars[key] : 
                this.trackVars[key]))
            // Add event to s.events, if it's true
            s.events += keys[k2];
          else
            // update value, fall back to options
            s[keys[k2]] = trackVars[key] !== undefined ?
              trackVars[key].toString() : 
              this.trackVars[key];
        }
      }
    }
  });

  videoTracker.registerTracker(tracker);
});
